/*
 * This file is part of SionTower.
 *
 * 
 * David Saltares Marquez (C) 2011
 * <david.saltares@gmail.com>
 *
 * 
 * SionTower examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * SionTower examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SionTower examples.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef SIONTOWER_TRUNK_SRC_INCLUDE_STATEGAME_H_
#define SIONTOWER_TRUNK_SRC_INCLUDE_STATEGAME_H_

#include <OGRE/Ogre.h>
#include <OIS/OIS.h>

#include "state.h"
#include "song.h"
#include "soundFX.h"
#include "sound3D.h"

#include "body.h"
#include "shape.h"
#include "player.h"

class StateManager;
class Level;

//! Clase que modela la din&aacute;mica de juego

/**
 *  @author David Saltares M&aacute;rquez
 *  @date 24-12-2010
 *
 *  StateManager es la clase que modela la jugabilidad de Sion Tower. Contiene
 *  el nivel, los enemigos y al personaje principal.
 */
class StateGame: public State{
    public:
        /**
         *  Constructor
         *
         *  @param stateManager gestor de estados asociado que controla el estado de
         *  juego.
         *
         *  Crea y carga todos los elementos del juego: protagonista, nivel,
         *  enemigo, GUI...
         */
        StateGame(StateManager* stateManager);

        /**
         *  Destructor
         *
         *  Destruye el estado de juego y todos sus elementos liberando la
         *  memoria.
         */
        ~StateGame();
        
        /**
         *  Carga los elementos del juego.
         */
        void load();

        /**
         *  Libera los elementos del juego.
         */
        void clear();

        /**
         *  @param deltaT tiempo en milisegundos desde la &uacute;ltima iteraci&oacute;n del
         *  bucle de juego.
         *  @param active true si el estado de juego es el estado activo en el
         *  gestor de estados (tope de la pila).
         *
         *  Actualiza todos los elementos del juego.
         */
        void update(Ogre::Real deltaT, bool active);


        /**
         *  Manejador del evento pulsar tecla
         *
         *  @param arg evento de tecla
         *  @return true si todo ha ido bien
         */
        bool keyPressed(const OIS::KeyEvent &arg);
        
        /**
         *  Manejador del evento soltar tecla
         *
         *  @param arg evento de tecla
         *  @return true si todo ha ido bien
         */
        bool keyReleased(const OIS::KeyEvent &arg);
        
        /**
         *  Manejador del evento mover rat&oacute;n
         *
         *  @param arg evento de rat&oacute;n
         *  @param true si todo ha ido bien
         */
        bool mouseMoved(const OIS::MouseEvent &arg);
        
        /**
         *  Manejador del evento pulsar bot&oacute;n de rat&oacute;n
         *
         *  @param arg evento de rat&oacute;n
         *  @param true si todo ha ido bien
         */
        bool mousePressed(const OIS::MouseEvent &arg, OIS::MouseButtonID id);
        
        /**
         *  Manejador del evento soltar bot&oacute;n de rat&oacute;n
         *
         *  @param arg evento de rat&oacute;n
         *  @param true si todo ha ido bien
         */
        bool mouseReleased(const OIS::MouseEvent &arg, OIS::MouseButtonID id);

        void beginCollisionCharacterWall(Body* bodyA, Body* bodyB);
        void inCollisionCharacterWall(Body* bodyA, Body* bodyB);
        void endCollisionCharacterWall(Body* bodyA, Body* bodyB);
    private:

        Ogre::Camera* _camera;
        OIS::Keyboard* _keyboard;

        SongPtr _song;
        Player* _player;

        Level* _level;
        
        void updateCamera(Ogre::Real deltaT);
};


#endif  // SIONTOWER_TRUNK_SRC_INCLUDE_STATEGAME_H_
